#include "mpris_connection.hpp"
#include <sdbus-c++/IConnection.h>
#include <sdbus-c++/Types.h>
#include <string>
#define IMGUI_DEFINE_MATH_OPERATORS
#include "imgui.h"

#include <curl/curl.h>
#include <fstream>
#include <iostream>
#include <magic.h>
#include <ostream>
#include <unordered_map>

MprisPlayer::MprisPlayer(std::string servicename) {
    bus_connection = sdbus::createSessionBusConnection();
    sdbus::ServiceName dest{servicename};
    sdbus::ObjectPath objec{"/org/mpris/MediaPlayer2"};
    image_path_cache = "NULL";
    PlayerProxy = sdbus::createProxy(dest, objec);
    mp2 = "org.mpris.MediaPlayer2.Player";
    Refresh();
}

void MprisPlayer::PausePlay() {
    auto method = PlayerProxy->createMethodCall(mp2, sdbus::MethodName("PlayPause"));
    try {
        auto reply = PlayerProxy->callMethod(method);
    } catch (const std::exception& e) {}
}

void MprisPlayer::Next() {
    auto method = PlayerProxy->createMethodCall(mp2, sdbus::MethodName("Next"));
    try {
        auto reply = PlayerProxy->callMethod(method);
    } catch (const std::exception &e) {}
}

void MprisPlayer::Prev() {
    auto method = PlayerProxy->createMethodCall(mp2, sdbus::MethodName("Previous"));
    try {
        auto reply = PlayerProxy->callMethod(method);
    } catch (const std::exception &e) {}
}

std::string MprisPlayer::GetIdentity() {
    return identity;
}

std::unordered_map<std::string, sdbus::Variant> MprisPlayer::GetCurrentMetadata() {
    return current_metadata;
}
void MprisPlayer::Refresh() {
    try {
        identity = PlayerProxy->getProperty("Identity")
                       .onInterface("org.mpris.MediaPlayer2")
                       .get<std::string>();
        playstate = PlayerProxy->getProperty("PlaybackStatus").onInterface("org.mpris.MediaPlayer2.Player").get<std::string>();
        current_metadata = PlayerProxy->getProperty("Metadata")
                               .onInterface("org.mpris.MediaPlayer2.Player")
                               .get<std::unordered_map<std::string, sdbus::Variant>>();
        if (current_metadata["mpris:artUrl"].get<std::string>() != image_path_cache) {
            image_path_cache = current_metadata["mpris:artUrl"].get<std::string>();
            UpdateTexture();
        }
    } catch (const std::exception& e) {}
    return;
}


static size_t WriteMemoryCallback(void* contents, size_t size, size_t nmemb,
                                  void* userp) {
    size_t realsize = size * nmemb;
    auto& mem = *static_cast<std::string*>(userp);
    mem.append(static_cast<char*>(contents), realsize);
    return realsize;
}

void MprisPlayer::UpdateTexture() {
    std::string filename = image_path_cache;
    std::cout << filename << std::endl;
    if (filename.length() > 0) {
        if (filename.c_str()[0] == 'h') {
            CURL* curl_handle;
            CURLcode res;

            std::string chunk;


            curl_handle = curl_easy_init();
            curl_easy_setopt(curl_handle, CURLOPT_URL, filename.c_str());
            curl_easy_setopt(curl_handle, CURLOPT_WRITEFUNCTION, WriteMemoryCallback);
            curl_easy_setopt(curl_handle, CURLOPT_WRITEDATA, &chunk);
            curl_easy_setopt(curl_handle, CURLOPT_USERAGENT, "libcurl-agent/1.0");

            // added options that may be required
            curl_easy_setopt(curl_handle, CURLOPT_FOLLOWLOCATION, 1L);  // redirects
            curl_easy_setopt(curl_handle, CURLOPT_HTTPPROXYTUNNEL, 1L); // corp. proxies etc.
            curl_easy_setopt(curl_handle, CURLOPT_VERBOSE, 1L); // we want it all
            // curl_easy_setopt(curl_handle, CURLOPT_REDIR_PROTOCOLS, CURLPROTO_HTTP | CURLPROTO_HTTPS);

            res = curl_easy_perform(curl_handle);

            if(res != CURLE_OK) {
                std::cerr << "curl_easy_perform() failed: " << curl_easy_strerror(res) << '\n';
            } else {
                std::cout << chunk.size() << " bytes retrieved\n";
            }

            std::ofstream output("/tmp/mpris-miniplayer-out");

            if (output.is_open()) {
                output << chunk;
                output.close();
            }
            filename = "file:///tmp/mpris-miniplayer-out";

            curl_easy_cleanup(curl_handle);


        }
        if (filename.c_str()[0] == 'f') {
            std::cout << "parsing as file" << std::endl;
            std::string path = filename.substr(7);
            FILE * f = fopen(path.c_str(), "r");
            if (f == NULL) {
                tex = RayTexture();
                fclose(f);
                return;
            }
            fseek(f, 0, SEEK_END);
            long size = ftell(f);
            rewind(f);
            unsigned char* buffer = new unsigned char[size];
            fread(buffer, size, 1, f);

            magic_t magic = magic_open(MAGIC_MIME_TYPE);
            if (magic == NULL) {
                tex = RayTexture();
                delete[] buffer;
                fclose(f);
                return;
            }

            if (magic_load(magic, NULL) != 0) {
                tex = RayTexture();
                delete[] buffer;
                fclose(f);
                return;
            }

            const char *mime_type = magic_file(magic, path.c_str());
            if (mime_type == NULL) {
                tex = RayTexture();
                delete[] buffer;
                fclose(f);
                return;
            }

            std::cout << mime_type << std::endl;
            if (strcmp(mime_type, "image/png") == 0 ) {
                auto img = LoadImageFromMemory(".png", buffer, size);
                tex = RayTexture(img);
                UnloadImage(img);
            } else if (strcmp(mime_type, "image/jpeg") == 0 ) {
                auto img = LoadImageFromMemory(".jpg", buffer, size);
                tex = RayTexture(img);
                UnloadImage(img);
            } else {
                tex = RayTexture();
            }
                delete[] buffer;
                fclose(f);
                return;

        }
    }
}

 RayTexture::RayTexture() {
    auto rtex = LoadRenderTexture(300,300);
    BeginTextureMode(rtex);
    DrawRectangle(0,0,300,300,BLACK);
    EndTextureMode();
    auto img = LoadImageFromTexture(rtex.texture);
    tex = LoadTextureFromImage(img);
    UnloadImage(img);
    UnloadRenderTexture(rtex);
    GenTextureMipmaps(&tex);
    SetTextureFilter(tex, TEXTURE_FILTER_BILINEAR);
    path = "null";
}

 RayTexture::RayTexture(Image img) {
    path = "NOPATH";
    tex = LoadTextureFromImage(img);
    GenTextureMipmaps(&tex);
    SetTextureFilter(tex, TEXTURE_FILTER_BILINEAR);
}

 RayTexture::RayTexture(const RayTexture &other) {
    path = other.path;
    auto img = LoadImageFromTexture(other.tex);
    tex = LoadTextureFromImage(img);
    GenTextureMipmaps(&tex);
    SetTextureFilter(tex, TEXTURE_FILTER_BILINEAR);
    UnloadImage(img);
}

RayTexture &RayTexture::operator=(const RayTexture &other) {
    if (this == &other) {
        return *this;
    }
    printf("HERE\n");
    RayTexture temp(other);
    std::swap(tex, temp.tex);
    std::swap(path, temp.path);
    return *this;
}

 RayTexture::~RayTexture() {
    if (IsWindowReady()) {
        UnloadTexture(tex);
    }
}
